@@ -1,4 +1,4 @@
-/**
+/*
  * Licensed to the Apache Software Foundation (ASF) under one
  * or more contributor license agreements.  See the NOTICE file
  * distributed with this work for additional information
@@ -19,56 +19,68 @@
 package org.apache.zookeeper.server;
 
 import static org.apache.zookeeper.test.ClientBase.CONNECTION_TIMEOUT;
+import static org.junit.Assert.assertEquals;
+import static org.junit.Assert.assertFalse;
+import static org.junit.Assert.assertNotNull;
+import static org.junit.Assert.assertTrue;
 import static org.junit.Assert.fail;
-
 import java.io.File;
 import java.io.FileWriter;
 import java.io.IOException;
 import java.util.concurrent.CountDownLatch;
 import java.util.concurrent.TimeUnit;
-
-import org.slf4j.Logger;
-import org.slf4j.LoggerFactory;
 import org.apache.zookeeper.CreateMode;
 import org.apache.zookeeper.KeeperException;
 import org.apache.zookeeper.PortAssignment;
 import org.apache.zookeeper.WatchedEvent;
 import org.apache.zookeeper.Watcher;
-import org.apache.zookeeper.ZKTestCase;
-import org.apache.zookeeper.ZooKeeper;
 import org.apache.zookeeper.Watcher.Event.KeeperState;
+import org.apache.zookeeper.ZKTestCase;
 import org.apache.zookeeper.ZooDefs.Ids;
+import org.apache.zookeeper.ZooKeeper;
 import org.apache.zookeeper.common.PathUtils;
+import org.apache.zookeeper.metrics.BaseTestMetricsProvider;
+import org.apache.zookeeper.metrics.BaseTestMetricsProvider.MetricsProviderCapturingLifecycle;
+import org.apache.zookeeper.metrics.BaseTestMetricsProvider.MetricsProviderWithConfiguration;
+import org.apache.zookeeper.metrics.BaseTestMetricsProvider.MetricsProviderWithErrorInConfigure;
+import org.apache.zookeeper.metrics.BaseTestMetricsProvider.MetricsProviderWithErrorInStart;
+import org.apache.zookeeper.metrics.BaseTestMetricsProvider.MetricsProviderWithErrorInStop;
 import org.apache.zookeeper.server.persistence.FileTxnSnapLog;
 import org.apache.zookeeper.server.quorum.QuorumPeerConfig.ConfigException;
 import org.apache.zookeeper.test.ClientBase;
-import org.junit.Assert;
 import org.junit.Test;
+import org.slf4j.Logger;
+import org.slf4j.LoggerFactory;
 
 /**
  * Test stand-alone server.
  *
  */
 public class ZooKeeperServerMainTest extends ZKTestCase implements Watcher {
-    protected static final Logger LOG =
-        LoggerFactory.getLogger(ZooKeeperServerMainTest.class);
+
+    protected static final Logger LOG = LoggerFactory.getLogger(ZooKeeperServerMainTest.class);
 
     private CountDownLatch clientConnected = new CountDownLatch(1);
 
     public static class MainThread extends Thread {
+
         final File confFile;
         final TestZKSMain main;
         final File tmpDir;
         final File dataDir;
         final File logDir;
 
-        public MainThread(int clientPort, boolean preCreateDirs, String configs)
-                throws IOException {
-            this(clientPort, preCreateDirs, ClientBase.createTmpDir(), configs);
+        public MainThread(int clientPort, boolean preCreateDirs, String configs) throws IOException {
+            this(clientPort, null, preCreateDirs, ClientBase.createTmpDir(), configs);
         }
 
-        public MainThread(int clientPort, boolean preCreateDirs, File tmpDir, String configs)
-                throws IOException {
+        public MainThread(int clientPort, Integer secureClientPort, boolean preCreateDirs, String configs)
+                throws  IOException {
+            this(clientPort, secureClientPort,
+                    preCreateDirs, ClientBase.createTmpDir(), configs);
+        }
+
+        public MainThread(int clientPort, Integer secureClientPort, boolean preCreateDirs, File tmpDir, String configs) throws IOException {
             super("Standalone server with clientPort:" + clientPort);
             this.tmpDir = tmpDir;
             confFile = new File(tmpDir, "zoo.cfg");
@@ -77,7 +89,7 @@ public class ZooKeeperServerMainTest extends ZKTestCase implements Watcher {
             fwriter.write("tickTime=2000\n");
             fwriter.write("initLimit=10\n");
             fwriter.write("syncLimit=5\n");
-            if(configs != null){
+            if (configs != null) {
                 fwriter.write(configs);
             }
 
@@ -90,13 +102,18 @@ public class ZooKeeperServerMainTest extends ZKTestCase implements Watcher {
                 if (!logDir.mkdir()) {
                     throw new IOException("unable to mkdir " + logDir);
                 }
+                ClientBase.createInitializeFile(logDir);
             }
-            
+
             String normalizedDataDir = PathUtils.normalizeFileSystemPath(dataDir.toString());
             String normalizedLogDir = PathUtils.normalizeFileSystemPath(logDir.toString());
             fwriter.write("dataDir=" + normalizedDataDir + "\n");
             fwriter.write("dataLogDir=" + normalizedLogDir + "\n");
             fwriter.write("clientPort=" + clientPort + "\n");
+
+            if (secureClientPort != null) {
+                fwriter.write("secureClientPort=" + secureClientPort + "\n");
+            }
             fwriter.flush();
             fwriter.close();
 
@@ -104,7 +121,7 @@ public class ZooKeeperServerMainTest extends ZKTestCase implements Watcher {
         }
 
         public void run() {
-            String args[] = new String[1];
+            String[] args = new String[1];
             args[0] = confFile.toString();
             try {
                 main.initializeAndRun(args);
@@ -118,31 +135,40 @@ public class ZooKeeperServerMainTest extends ZKTestCase implements Watcher {
             main.shutdown();
         }
 
-        void deleteDirs() throws IOException{
+        void deleteDirs() throws IOException {
             delete(tmpDir);
         }
 
         void delete(File f) throws IOException {
             if (f.isDirectory()) {
-                for (File c : f.listFiles())
+                for (File c : f.listFiles()) {
                     delete(c);
+                }
             }
-            if (!f.delete())
+            if (!f.delete()) {
                 // double check for the file existence
                 if (f.exists()) {
                     throw new IOException("Failed to delete file: " + f);
                 }
+            }
         }
 
         ServerCnxnFactory getCnxnFactory() {
             return main.getCnxnFactory();
         }
+
+        public ServerCnxnFactory getSecureCnxnFactory(){
+            return main.getSecureCnxnFactory();
+        }
+
     }
 
-    public static  class TestZKSMain extends ZooKeeperServerMain {
+    public static class TestZKSMain extends ZooKeeperServerMain {
+
         public void shutdown() {
             super.shutdown();
         }
+
     }
 
     /**
@@ -159,24 +185,19 @@ public class ZooKeeperServerMainTest extends ZKTestCase implements Watcher {
         MainThread main = new MainThread(CLIENT_PORT, true, null);
         main.start();
 
-        Assert.assertTrue("waiting for server being up",
-                ClientBase.waitForServerUp("127.0.0.1:" + CLIENT_PORT,
-                        CONNECTION_TIMEOUT));
+        assertTrue(
+                "waiting for server being up",
+                ClientBase.waitForServerUp("127.0.0.1:" + CLIENT_PORT, CONNECTION_TIMEOUT));
 
+        ZooKeeper zk = new ZooKeeper("127.0.0.1:" + CLIENT_PORT, ClientBase.CONNECTION_TIMEOUT, this);
 
-        ZooKeeper zk = new ZooKeeper("127.0.0.1:" + CLIENT_PORT,
-                ClientBase.CONNECTION_TIMEOUT, this);
-
-        zk.create("/foo1", "foobar".getBytes(), Ids.OPEN_ACL_UNSAFE,
-                CreateMode.PERSISTENT);
-        Assert.assertEquals(new String(zk.getData("/foo1", null, null)), "foobar");
+        zk.create("/foo1", "foobar".getBytes(), Ids.OPEN_ACL_UNSAFE, CreateMode.PERSISTENT);
+        assertEquals(new String(zk.getData("/foo1", null, null)), "foobar");
 
         // inject problem in server
-        ZooKeeperServer zooKeeperServer = main.getCnxnFactory()
-                .getZooKeeperServer();
+        ZooKeeperServer zooKeeperServer = main.getCnxnFactory().getZooKeeperServer();
         FileTxnSnapLog snapLog = zooKeeperServer.getTxnLogFactory();
-        FileTxnSnapLog fileTxnSnapLogWithError = new FileTxnSnapLog(
-                snapLog.getDataDir(), snapLog.getSnapDir()) {
+        FileTxnSnapLog fileTxnSnapLogWithError = new FileTxnSnapLog(snapLog.getDataDir(), snapLog.getSnapDir()) {
             @Override
             public void commit() throws IOException {
                 throw new IOException("Input/output error");
@@ -187,16 +208,15 @@ public class ZooKeeperServerMainTest extends ZKTestCase implements Watcher {
 
         try {
             // do create operation, so that injected IOException is thrown
-            zk.create("/foo2", "foobar".getBytes(), Ids.OPEN_ACL_UNSAFE,
-                    CreateMode.PERSISTENT);
+            zk.create("/foo2", "foobar".getBytes(), Ids.OPEN_ACL_UNSAFE, CreateMode.PERSISTENT);
             fail("IOException is expected as error is injected in transaction log commit funtionality");
         } catch (Exception e) {
             // do nothing
         }
         zk.close();
-        Assert.assertTrue("waiting for server down",
-                ClientBase.waitForServerDown("127.0.0.1:" + CLIENT_PORT,
-                        ClientBase.CONNECTION_TIMEOUT));
+        assertTrue(
+                "waiting for server down",
+                ClientBase.waitForServerDown("127.0.0.1:" + CLIENT_PORT, ClientBase.CONNECTION_TIMEOUT));
         fileTxnSnapLogWithError.close();
         main.shutdown();
         main.deleteDirs();
@@ -218,9 +238,9 @@ public class ZooKeeperServerMainTest extends ZKTestCase implements Watcher {
         MainThread main = new MainThread(CLIENT_PORT, true, null);
         File tmpDir = main.tmpDir;
         main.start();
-        Assert.assertTrue("waiting for server being up", ClientBase
-                .waitForServerUp("127.0.0.1:" + CLIENT_PORT,
-                        CONNECTION_TIMEOUT / 2));
+        assertTrue(
+                "waiting for server being up",
+                ClientBase.waitForServerUp("127.0.0.1:" + CLIENT_PORT, CONNECTION_TIMEOUT / 2));
         main.shutdown();
 
         // Make the snapshot directory read only
@@ -228,12 +248,12 @@ public class ZooKeeperServerMainTest extends ZKTestCase implements Watcher {
         snapDir.setWritable(false);
 
         // Restart ZK and observe a failure
-        main = new MainThread(CLIENT_PORT, false, tmpDir, null);
+        main = new MainThread(CLIENT_PORT, null, false, tmpDir, null);
         main.start();
 
-        Assert.assertFalse("waiting for server being up", ClientBase
-                .waitForServerUp("127.0.0.1:" + CLIENT_PORT,
-                        CONNECTION_TIMEOUT / 2));
+        assertFalse(
+                "waiting for server being up",
+                ClientBase.waitForServerUp("127.0.0.1:" + CLIENT_PORT, CONNECTION_TIMEOUT / 2));
 
         main.shutdown();
 
@@ -258,9 +278,9 @@ public class ZooKeeperServerMainTest extends ZKTestCase implements Watcher {
         MainThread main = new MainThread(CLIENT_PORT, true, null);
         File tmpDir = main.tmpDir;
         main.start();
-        Assert.assertTrue("waiting for server being up", ClientBase
-                .waitForServerUp("127.0.0.1:" + CLIENT_PORT,
-                        CONNECTION_TIMEOUT / 2));
+        assertTrue(
+                "waiting for server being up",
+                ClientBase.waitForServerUp("127.0.0.1:" + CLIENT_PORT, CONNECTION_TIMEOUT / 2));
         main.shutdown();
 
         // Make the transaction log directory read only
@@ -268,12 +288,12 @@ public class ZooKeeperServerMainTest extends ZKTestCase implements Watcher {
         logDir.setWritable(false);
 
         // Restart ZK and observe a failure
-        main = new MainThread(CLIENT_PORT, false, tmpDir, null);
+        main = new MainThread(CLIENT_PORT, null, false, tmpDir, null);
         main.start();
 
-        Assert.assertFalse("waiting for server being up", ClientBase
-                .waitForServerUp("127.0.0.1:" + CLIENT_PORT,
-                        CONNECTION_TIMEOUT / 2));
+        assertFalse(
+                "waiting for server being up",
+                ClientBase.waitForServerUp("127.0.0.1:" + CLIENT_PORT, CONNECTION_TIMEOUT / 2));
 
         main.shutdown();
 
@@ -294,122 +314,230 @@ public class ZooKeeperServerMainTest extends ZKTestCase implements Watcher {
         MainThread main = new MainThread(CLIENT_PORT, true, null);
         main.start();
 
-        Assert.assertTrue("waiting for server being up",
-                ClientBase.waitForServerUp("127.0.0.1:" + CLIENT_PORT,
-                        CONNECTION_TIMEOUT));
+        assertTrue(
+                "waiting for server being up",
+                ClientBase.waitForServerUp("127.0.0.1:" + CLIENT_PORT, CONNECTION_TIMEOUT));
 
         clientConnected = new CountDownLatch(1);
-        ZooKeeper zk = new ZooKeeper("127.0.0.1:" + CLIENT_PORT,
-                ClientBase.CONNECTION_TIMEOUT, this);
-        Assert.assertTrue("Failed to establish zkclient connection!",
-                clientConnected.await(CONNECTION_TIMEOUT, TimeUnit.MILLISECONDS));
+        ZooKeeper zk = new ZooKeeper("127.0.0.1:" + CLIENT_PORT, ClientBase.CONNECTION_TIMEOUT, this);
+        assertTrue("Failed to establish zkclient connection!", clientConnected.await(CONNECTION_TIMEOUT, TimeUnit.MILLISECONDS));
 
-        zk.create("/foo", "foobar".getBytes(), Ids.OPEN_ACL_UNSAFE,
-                CreateMode.PERSISTENT);
-        Assert.assertEquals(new String(zk.getData("/foo", null, null)), "foobar");
+        zk.create("/foo", "foobar".getBytes(), Ids.OPEN_ACL_UNSAFE, CreateMode.PERSISTENT);
+        assertEquals(new String(zk.getData("/foo", null, null)), "foobar");
         zk.close();
 
         main.shutdown();
         main.join();
         main.deleteDirs();
 
-        Assert.assertTrue("waiting for server down",
-                ClientBase.waitForServerDown("127.0.0.1:" + CLIENT_PORT,
-                        ClientBase.CONNECTION_TIMEOUT));
+        assertTrue(
+                "waiting for server down",
+                ClientBase.waitForServerDown("127.0.0.1:" + CLIENT_PORT, ClientBase.CONNECTION_TIMEOUT));
     }
 
     /**
-     * Test verifies server should fail when data dir or data log dir doesn't
-     * exists. Sets "zookeeper.datadir.autocreate" to false.
-     */
-    @Test(timeout = 30000)
-    public void testWithoutAutoCreateDataLogDir() throws Exception {
-        ClientBase.setupTestEnv();
-        System.setProperty(FileTxnSnapLog.ZOOKEEPER_DATADIR_AUTOCREATE, "false");
-        try {
-            final int CLIENT_PORT = PortAssignment.unique();
-
-            MainThread main = new MainThread(CLIENT_PORT, false, null);
-            String args[] = new String[1];
-            args[0] = main.confFile.toString();
-            main.start();
-
-            Assert.assertFalse("waiting for server being up", ClientBase
-                    .waitForServerUp("127.0.0.1:" + CLIENT_PORT,
-                            CONNECTION_TIMEOUT / 2));
-        } finally {
-            // resets "zookeeper.datadir.autocreate" flag
-            System.setProperty(FileTxnSnapLog.ZOOKEEPER_DATADIR_AUTOCREATE,
-                    FileTxnSnapLog.ZOOKEEPER_DATADIR_AUTOCREATE_DEFAULT);
-        }
-    }
-
-    /**
-     * Test verifies the auto creation of data dir and data log dir.
-     * Sets "zookeeper.datadir.autocreate" to true.
-     */
-    @Test(timeout = 30000)
-    public void testWithAutoCreateDataLogDir() throws Exception {
-        ClientBase.setupTestEnv();
-        System.setProperty(FileTxnSnapLog.ZOOKEEPER_DATADIR_AUTOCREATE, "true");
-        final int CLIENT_PORT = PortAssignment.unique();
-
-        MainThread main = new MainThread(CLIENT_PORT, false, null);
-        String args[] = new String[1];
-        args[0] = main.confFile.toString();
-        main.start();
-
-        Assert.assertTrue("waiting for server being up",
-                ClientBase.waitForServerUp("127.0.0.1:" + CLIENT_PORT,
-                        CONNECTION_TIMEOUT));
-        clientConnected = new CountDownLatch(1);
-        ZooKeeper zk = new ZooKeeper("127.0.0.1:" + CLIENT_PORT,
-                ClientBase.CONNECTION_TIMEOUT, this);
-        Assert.assertTrue("Failed to establish zkclient connection!",
-                clientConnected.await(CONNECTION_TIMEOUT, TimeUnit.MILLISECONDS));
-
-        zk.create("/foo", "foobar".getBytes(), Ids.OPEN_ACL_UNSAFE,
-                CreateMode.PERSISTENT);
-        Assert.assertEquals(new String(zk.getData("/foo", null, null)),
-                "foobar");
-        zk.close();
-
-        main.shutdown();
-        main.join();
-        main.deleteDirs();
-
-        Assert.assertTrue("waiting for server down", ClientBase
-                .waitForServerDown("127.0.0.1:" + CLIENT_PORT,
-                        ClientBase.CONNECTION_TIMEOUT));
-    }
-
-    /**
-     * Test verifies that the server shouldn't allow minsessiontimeout >
+     * Test verifies that the server shouldn't allow minsessiontimeout greater than
      * maxsessiontimeout
      */
     @Test
-    public void testWithMinSessionTimeoutGreaterThanMaxSessionTimeout()
-            throws Exception {
+    public void testWithMinSessionTimeoutGreaterThanMaxSessionTimeout() throws Exception {
         ClientBase.setupTestEnv();
 
         final int CLIENT_PORT = PortAssignment.unique();
         final int tickTime = 2000;
         final int minSessionTimeout = 20 * tickTime + 1000; // min is higher
         final int maxSessionTimeout = tickTime * 2 - 100; // max is lower
-        final String configs = "maxSessionTimeout=" + maxSessionTimeout + "\n"
-                + "minSessionTimeout=" + minSessionTimeout + "\n";
-        MainThread main = new MainThread(CLIENT_PORT, false, configs);
-        String args[] = new String[1];
+        final String configs = "maxSessionTimeout="
+                                       + maxSessionTimeout
+                                       + "\n"
+                                       + "minSessionTimeout="
+                                       + minSessionTimeout
+                                       + "\n";
+        MainThread main = new MainThread(CLIENT_PORT, true, configs);
+        String[] args = new String[1];
         args[0] = main.confFile.toString();
         try {
             main.main.initializeAndRun(args);
-            Assert.fail("Must throw exception as "
-                    + "minsessiontimeout > maxsessiontimeout");
+            fail("Must throw exception as " + "minsessiontimeout > maxsessiontimeout");
         } catch (ConfigException iae) {
             // expected
         }
     }
 
+    /**
+     * Test verifies that the server shouldn't boot with an invalid metrics provider
+     */
+    @Test
+    public void testInvalidMetricsProvider() throws Exception {
+        ClientBase.setupTestEnv();
+
+        final int CLIENT_PORT = PortAssignment.unique();
+        final String configs = "metricsProvider.className=BadClass\n";
+        MainThread main = new MainThread(CLIENT_PORT, true, configs);
+        String[] args = new String[1];
+        args[0] = main.confFile.toString();
+        try {
+            main.main.initializeAndRun(args);
+            fail("Must throw exception as metrics provider is not " + "well configured");
+        } catch (ConfigException iae) {
+            // expected
+        }
+    }
+
+    /**
+     * Test verifies that the server shouldn't boot with a faulty metrics provider
+     */
+    @Test
+    public void testFaultyMetricsProviderOnStart() throws Exception {
+        ClientBase.setupTestEnv();
+
+        final int CLIENT_PORT = PortAssignment.unique();
+        final String configs = "metricsProvider.className=" + MetricsProviderWithErrorInStart.class.getName() + "\n";
+        MainThread main = new MainThread(CLIENT_PORT, true, configs);
+        String[] args = new String[1];
+        args[0] = main.confFile.toString();
+        try {
+            main.main.initializeAndRun(args);
+            fail("Must throw exception as metrics provider cannot boot");
+        } catch (IOException iae) {
+            // expected
+        }
+    }
+
+    /**
+     * Test verifies that the server shouldn't boot with a faulty metrics provider
+     */
+    @Test
+    public void testFaultyMetricsProviderOnConfigure() throws Exception {
+        ClientBase.setupTestEnv();
+
+        final int CLIENT_PORT = PortAssignment.unique();
+        final String configs = "metricsProvider.className="
+                                       + MetricsProviderWithErrorInConfigure.class.getName()
+                                       + "\n";
+        MainThread main = new MainThread(CLIENT_PORT, true, configs);
+        String[] args = new String[1];
+        args[0] = main.confFile.toString();
+        try {
+            main.main.initializeAndRun(args);
+            fail("Must throw exception as metrics provider is cannot boot");
+        } catch (IOException iae) {
+            // expected
+        }
+    }
+
+    /**
+     * Test verifies that the server shouldn't be affected but runtime errors on stop()
+     */
+    @Test
+    public void testFaultyMetricsProviderOnStop() throws Exception {
+        ClientBase.setupTestEnv();
+
+        final int CLIENT_PORT = PortAssignment.unique();
+        MetricsProviderWithErrorInStop.stopCalled.set(false);
+        final String configs = "metricsProvider.className=" + MetricsProviderWithErrorInStop.class.getName() + "\n";
+        MainThread main = new MainThread(CLIENT_PORT, true, configs);
+        main.start();
+
+        assertTrue(
+                "waiting for server being up",
+                ClientBase.waitForServerUp("127.0.0.1:" + CLIENT_PORT, CONNECTION_TIMEOUT));
+
+        clientConnected = new CountDownLatch(1);
+        ZooKeeper zk = new ZooKeeper("127.0.0.1:" + CLIENT_PORT, ClientBase.CONNECTION_TIMEOUT, this);
+        assertTrue("Failed to establish zkclient connection!", clientConnected.await(CONNECTION_TIMEOUT, TimeUnit.MILLISECONDS));
+
+        zk.create("/foo", "foobar".getBytes(), Ids.OPEN_ACL_UNSAFE, CreateMode.PERSISTENT);
+        assertEquals(new String(zk.getData("/foo", null, null)), "foobar");
+        zk.close();
+
+        main.shutdown();
+        main.join();
+        main.deleteDirs();
+
+        assertTrue(
+                "waiting for server down",
+                ClientBase.waitForServerDown("127.0.0.1:" + CLIENT_PORT, ClientBase.CONNECTION_TIMEOUT));
+        assertTrue(MetricsProviderWithErrorInStop.stopCalled.get());
+    }
+
+    /**
+     * Test verifies that configuration is passed to the MetricsProvider.
+     */
+    @Test
+    public void testMetricsProviderConfiguration() throws Exception {
+        ClientBase.setupTestEnv();
+
+        final int CLIENT_PORT = PortAssignment.unique();
+        MetricsProviderWithConfiguration.httpPort.set(0);
+        final String configs = "metricsProvider.className="
+                                       + MetricsProviderWithConfiguration.class.getName()
+                                       + "\n"
+                                       + "metricsProvider.httpPort=1234\n";
+        MainThread main = new MainThread(CLIENT_PORT, true, configs);
+        main.start();
+
+        assertTrue(
+                "waiting for server being up",
+                ClientBase.waitForServerUp("127.0.0.1:" + CLIENT_PORT, CONNECTION_TIMEOUT));
+
+        clientConnected = new CountDownLatch(1);
+        ZooKeeper zk = new ZooKeeper("127.0.0.1:" + CLIENT_PORT, ClientBase.CONNECTION_TIMEOUT, this);
+        assertTrue("Failed to establish zkclient connection!", clientConnected.await(CONNECTION_TIMEOUT, TimeUnit.MILLISECONDS));
+
+        zk.create("/foo", "foobar".getBytes(), Ids.OPEN_ACL_UNSAFE, CreateMode.PERSISTENT);
+        assertEquals(new String(zk.getData("/foo", null, null)), "foobar");
+        zk.close();
+
+        main.shutdown();
+        main.join();
+        main.deleteDirs();
+
+        assertTrue(
+                "waiting for server down",
+                ClientBase.waitForServerDown("127.0.0.1:" + CLIENT_PORT, ClientBase.CONNECTION_TIMEOUT));
+        assertEquals(1234, MetricsProviderWithConfiguration.httpPort.get());
+    }
+
+    /**
+     * Test verifies that all of the lifecycle methods of the MetricsProvider are called.
+     */
+    @Test
+    public void testMetricsProviderLifecycle() throws Exception {
+        ClientBase.setupTestEnv();
+        MetricsProviderCapturingLifecycle.reset();
+
+        final int CLIENT_PORT = PortAssignment.unique();
+        final String configs = "metricsProvider.className="
+                                       + MetricsProviderCapturingLifecycle.class.getName()
+                                       + "\n"
+                                       + "metricsProvider.httpPort=1234\n";
+        MainThread main = new MainThread(CLIENT_PORT, true, configs);
+        main.start();
+
+        assertTrue(
+                "waiting for server being up",
+                ClientBase.waitForServerUp("127.0.0.1:" + CLIENT_PORT, CONNECTION_TIMEOUT));
+
+        clientConnected = new CountDownLatch(1);
+        ZooKeeper zk = new ZooKeeper("127.0.0.1:" + CLIENT_PORT, ClientBase.CONNECTION_TIMEOUT, this);
+        assertTrue("Failed to establish zkclient connection!", clientConnected.await(CONNECTION_TIMEOUT, TimeUnit.MILLISECONDS));
+
+        zk.create("/foo", "foobar".getBytes(), Ids.OPEN_ACL_UNSAFE, CreateMode.PERSISTENT);
+        assertEquals(new String(zk.getData("/foo", null, null)), "foobar");
+        zk.close();
+
+        main.shutdown();
+        main.join();
+        main.deleteDirs();
+
+        assertTrue("waiting for server down", ClientBase.waitForServerDown("127.0.0.1:" + CLIENT_PORT, ClientBase.CONNECTION_TIMEOUT));
+        assertTrue("metrics provider lifecycle error", BaseTestMetricsProvider.MetricsProviderCapturingLifecycle.configureCalled.get());
+        assertTrue("metrics provider lifecycle error", BaseTestMetricsProvider.MetricsProviderCapturingLifecycle.startCalled.get());
+        assertTrue("metrics provider lifecycle error", BaseTestMetricsProvider.MetricsProviderCapturingLifecycle.getRootContextCalled.get());
+        assertTrue("metrics provider lifecycle error", BaseTestMetricsProvider.MetricsProviderCapturingLifecycle.stopCalled.get());
+    }
+
     /**
      * Test verifies that the server is able to redefine if user configured only
      * minSessionTimeout limit
@@ -423,23 +551,19 @@ public class ZooKeeperServerMainTest extends ZKTestCase implements Watcher {
         final int minSessionTimeout = tickTime * 2 - 100;
         int maxSessionTimeout = 20 * tickTime;
         final String configs = "minSessionTimeout=" + minSessionTimeout + "\n";
-        MainThread main = new MainThread(CLIENT_PORT, false, configs);
+        MainThread main = new MainThread(CLIENT_PORT, true, configs);
         main.start();
 
         String HOSTPORT = "127.0.0.1:" + CLIENT_PORT;
-        Assert.assertTrue("waiting for server being up",
-                ClientBase.waitForServerUp(HOSTPORT, CONNECTION_TIMEOUT));
+        assertTrue("waiting for server being up", ClientBase.waitForServerUp(HOSTPORT, CONNECTION_TIMEOUT));
         // create session with min value
         verifySessionTimeOut(minSessionTimeout, minSessionTimeout, HOSTPORT);
-        verifySessionTimeOut(minSessionTimeout - 2000, minSessionTimeout,
-                HOSTPORT);
+        verifySessionTimeOut(minSessionTimeout - 2000, minSessionTimeout, HOSTPORT);
         // create session with max value
         verifySessionTimeOut(maxSessionTimeout, maxSessionTimeout, HOSTPORT);
-        verifySessionTimeOut(maxSessionTimeout + 2000, maxSessionTimeout,
-                HOSTPORT);
+        verifySessionTimeOut(maxSessionTimeout + 2000, maxSessionTimeout, HOSTPORT);
         main.shutdown();
-        Assert.assertTrue("waiting for server down", ClientBase
-                .waitForServerDown(HOSTPORT, ClientBase.CONNECTION_TIMEOUT));
+        assertTrue("waiting for server down", ClientBase.waitForServerDown(HOSTPORT, ClientBase.CONNECTION_TIMEOUT));
     }
 
     /**
@@ -454,37 +578,33 @@ public class ZooKeeperServerMainTest extends ZKTestCase implements Watcher {
         final int tickTime = 2000;
         final int minSessionTimeout = tickTime * 2 - 100;
         final int maxSessionTimeout = 20 * tickTime + 1000;
-        final String configs = "maxSessionTimeout=" + maxSessionTimeout + "\n"
-                + "minSessionTimeout=" + minSessionTimeout + "\n";
-        MainThread main = new MainThread(CLIENT_PORT, false, configs);
+        final String configs = "maxSessionTimeout="
+                                       + maxSessionTimeout
+                                       + "\n"
+                                       + "minSessionTimeout="
+                                       + minSessionTimeout
+                                       + "\n";
+        MainThread main = new MainThread(CLIENT_PORT, true, configs);
         main.start();
 
         String HOSTPORT = "127.0.0.1:" + CLIENT_PORT;
-        Assert.assertTrue("waiting for server being up",
-                ClientBase.waitForServerUp(HOSTPORT, CONNECTION_TIMEOUT));
+        assertTrue("waiting for server being up", ClientBase.waitForServerUp(HOSTPORT, CONNECTION_TIMEOUT));
         // create session with min value
         verifySessionTimeOut(minSessionTimeout, minSessionTimeout, HOSTPORT);
-        verifySessionTimeOut(minSessionTimeout - 2000, minSessionTimeout,
-                HOSTPORT);
+        verifySessionTimeOut(minSessionTimeout - 2000, minSessionTimeout, HOSTPORT);
         // create session with max value
         verifySessionTimeOut(maxSessionTimeout, maxSessionTimeout, HOSTPORT);
-        verifySessionTimeOut(maxSessionTimeout + 2000, maxSessionTimeout,
-                HOSTPORT);
+        verifySessionTimeOut(maxSessionTimeout + 2000, maxSessionTimeout, HOSTPORT);
         main.shutdown();
 
-        Assert.assertTrue("waiting for server down", ClientBase
-                .waitForServerDown(HOSTPORT, ClientBase.CONNECTION_TIMEOUT));
+        assertTrue("waiting for server down", ClientBase.waitForServerDown(HOSTPORT, ClientBase.CONNECTION_TIMEOUT));
     }
 
-    private void verifySessionTimeOut(int sessionTimeout,
-            int expectedSessionTimeout, String HOSTPORT) throws IOException,
-            KeeperException, InterruptedException {
+    private void verifySessionTimeOut(int sessionTimeout, int expectedSessionTimeout, String HOSTPORT) throws IOException, KeeperException, InterruptedException {
         clientConnected = new CountDownLatch(1);
         ZooKeeper zk = new ZooKeeper(HOSTPORT, sessionTimeout, this);
-        Assert.assertTrue("Failed to establish zkclient connection!",
-                clientConnected.await(sessionTimeout, TimeUnit.MILLISECONDS));
-        Assert.assertEquals("Not able to configure the sessionTimeout values",
-                expectedSessionTimeout, zk.getSessionTimeout());
+        assertTrue("Failed to establish zkclient connection!", clientConnected.await(sessionTimeout, TimeUnit.MILLISECONDS));
+        assertEquals("Not able to configure the sessionTimeout values", expectedSessionTimeout, zk.getSessionTimeout());
         zk.close();
     }
 
@@ -505,10 +625,8 @@ public class ZooKeeperServerMainTest extends ZKTestCase implements Watcher {
 
     @Test
     public void testJMXRegistrationWithNetty() throws Exception {
-        String originalServerCnxnFactory = System
-                .getProperty(ServerCnxnFactory.ZOOKEEPER_SERVER_CNXN_FACTORY);
-        System.setProperty(ServerCnxnFactory.ZOOKEEPER_SERVER_CNXN_FACTORY,
-                NettyServerCnxnFactory.class.getName());
+        String originalServerCnxnFactory = System.getProperty(ServerCnxnFactory.ZOOKEEPER_SERVER_CNXN_FACTORY);
+        System.setProperty(ServerCnxnFactory.ZOOKEEPER_SERVER_CNXN_FACTORY, NettyServerCnxnFactory.class.getName());
         try {
             ClientBase.setupTestEnv();
             File tmpDir_1 = ClientBase.createTmpDir();
@@ -523,39 +641,38 @@ public class ZooKeeperServerMainTest extends ZKTestCase implements Watcher {
             deleteFile(tmpDir_2);
         } finally {
             // setting back
-            if (originalServerCnxnFactory == null
-                    || originalServerCnxnFactory.isEmpty()) {
+            if (originalServerCnxnFactory == null || originalServerCnxnFactory.isEmpty()) {
                 System.clearProperty(ServerCnxnFactory.ZOOKEEPER_SERVER_CNXN_FACTORY);
             } else {
-                System.setProperty(
-                        ServerCnxnFactory.ZOOKEEPER_SERVER_CNXN_FACTORY,
-                        originalServerCnxnFactory);
+                System.setProperty(ServerCnxnFactory.ZOOKEEPER_SERVER_CNXN_FACTORY, originalServerCnxnFactory);
             }
         }
     }
 
     private void deleteFile(File f) throws IOException {
         if (f.isDirectory()) {
-            for (File c : f.listFiles())
+            for (File c : f.listFiles()) {
                 deleteFile(c);
+            }
         }
-        if (!f.delete())
-            // double check for the file existence
+        if (!f.delete()) {
+        // double check for the file existence
+
             if (f.exists()) {
                 throw new IOException("Failed to delete file: " + f);
             }
+        }
     }
 
-    private ServerCnxnFactory startServer(File tmpDir) throws IOException,
-            InterruptedException {
+    private ServerCnxnFactory startServer(File tmpDir) throws IOException, InterruptedException {
         final int CLIENT_PORT = PortAssignment.unique();
         ZooKeeperServer zks = new ZooKeeperServer(tmpDir, tmpDir, 3000);
         ServerCnxnFactory f = ServerCnxnFactory.createFactory(CLIENT_PORT, -1);
         f.startup(zks);
-        Assert.assertNotNull("JMX initialization failed!", zks.jmxServerBean);
-        Assert.assertTrue("waiting for server being up",
-                ClientBase.waitForServerUp("127.0.0.1:" + CLIENT_PORT,
-                        CONNECTION_TIMEOUT));
+        assertNotNull("JMX initialization failed!", zks.jmxServerBean);
+        assertTrue(
+                "waiting for server being up",
+                ClientBase.waitForServerUp("127.0.0.1:" + CLIENT_PORT, CONNECTION_TIMEOUT));
         return f;
     }
 
@@ -564,4 +681,5 @@ public class ZooKeeperServerMainTest extends ZKTestCase implements Watcher {
             clientConnected.countDown();
         }
     }
+
 }
